@@ -1,4 +1,4 @@
-set rcsId {$Id: boxlabel.tcl,v 1.39 1998/06/07 10:20:43 jfontain Exp $}
+set rcsId {$Id: boxlabel.tcl,v 1.41 1999/03/27 21:41:06 jfontain Exp $}
 
 class pieBoxLabeler {
 
@@ -8,7 +8,7 @@ class pieBoxLabeler {
     }
 
     proc ~pieBoxLabeler {this} {
-        catch {::delete $pieBoxLabeler::($this,array)}                                        ;# array may not have been created yet
+        ::delete $pieBoxLabeler::($this,array)
     }
 
     proc options {this} {                                      ;# font and justify options are used when creating a new canvas label
@@ -55,11 +55,7 @@ class pieBoxLabeler {
         if {[string length $selected]==0} {                                                   ;# return current state if no argument
             return $pieBoxLabeler::($this,selected,$label)
         }
-        if {$selected} {
-            switched::configure $label -borderwidth 2
-        } else {
-            switched::configure $label -borderwidth 1
-        }
+        switched::configure $label -select $selected
         ::set pieBoxLabeler::($this,selected,$label) $selected
     }
 
