@@ -1,10 +1,10 @@
 
-set rcsId {$Id: demo,v 1.39 1998/06/09 21:11:45 jfontain Exp $}
+set rcsId {$Id: demo,v 1.42 1999/03/30 21:19:32 jfontain Exp $}
 
-if {[catch {package require stooop 3.6}]} {
-set rcsId {$Id: stooop.tcl,v 3.62 1998/06/05 18:29:09 jfontain Exp $}
+if {[catch {package require stooop 3.7}]} {
+set rcsId {$Id: stooop.tcl,v 3.66 1998/10/28 21:50:15 jfontain Exp $}
 
-package provide stooop 3.6
+package provide stooop 3.7
 
 catch {rename proc _proc}
 
@@ -18,7 +18,7 @@ namespace eval ::stooop {
 
     set checkCode {}
     if {[info exists ::env(STOOOPCHECKALL)]} {
-        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {}}
+        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {} STOOOPCHECKOBJECTS {}}
     }
     if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
         append checkCode {::stooop::checkProcedure;}
@@ -387,24 +387,51 @@ if {[llength [array names ::env STOOOP*]]>0} {
                 set interface($fullClass) {}
             }
         }
+    }
 
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc invokingProcedure {} {
+            if {[catch {set procedure [lindex [info level -2] 0]}]} {
+                return {main script}
+            } elseif {[string length $procedure]==0} {
+                return "namespace [uplevel 2 namespace current]"
+            } else {
+                return [uplevel 3 namespace which -command $procedure]
+            }
+        }
+    }
+
+    if {[info exists ::env(STOOOPCHECKPROCEDURES)]||[info exists ::env(STOOOPCHECKOBJECTS)]} {
         catch {rename ::stooop::new ::stooop::_new}
         _proc ::stooop::new {classOrId args} {
-            variable fullClass
-            variable interface
-
-            if {[scan $classOrId %u dummy]==0} {
-                set constructor ${classOrId}::[namespace tail $classOrId]
-                catch {$constructor}
-                set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            } else {
-                set fullName $fullClass($classOrId)
+            variable newId
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                variable fullClass
+                variable interface
             }
-            if {[info exists interface($fullName)]} {
-                error "class $fullName with pure virtual procedures should not be instanciated"
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                variable creator
+            }
+
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                if {[scan $classOrId %u dummy]==0} {
+                    set constructor ${classOrId}::[namespace tail $classOrId]
+                    catch {$constructor}
+                    set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
+                    set fullClass([expr {$newId+1}]) $fullName
+                } else {
+                    set fullName $fullClass($classOrId)
+                }
+                if {[info exists interface($fullName)]} {
+                    error "class $fullName with pure virtual procedures should not be instanciated"
+                }
+            }
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                set creator([expr {$newId+1}]) [invokingProcedure]
             }
             return [uplevel ::stooop::_new $classOrId $args]
         }
+
     }
 
     _proc ::stooop::ancestors {fullClass} {
@@ -545,6 +572,50 @@ if {[llength [array names ::env STOOOP*]]>0} {
         }
         puts $traceDataChannel $text
     }
+
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc ::stooop::printObjects {{pattern *}} {
+            variable fullClass
+            variable creator
+
+            puts "stooop::printObjects invoked from [invokingProcedure]:"
+            foreach id [lsort -integer [array names fullClass]] {
+                if {[string match $pattern $fullClass($id)]} {
+                    puts "$fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+
+        _proc ::stooop::record {} {
+            variable fullClass
+            variable checkpointFullClass
+
+            puts "stooop::record invoked from [invokingProcedure]"
+            catch {unset checkpointFullClass}
+            array set checkpointFullClass [array get fullClass]
+        }
+
+        _proc ::stooop::report {{pattern *}} {
+            variable fullClass
+            variable checkpointFullClass
+            variable creator
+
+            puts "stooop::report invoked from [invokingProcedure]:"
+            set checkpointIds [lsort -integer [array names checkpointFullClass]]
+            set currentIds [lsort -integer [array names fullClass]]
+            foreach id $currentIds {
+                if {[string match $pattern $fullClass($id)]&&([lsearch -exact $checkpointIds $id]<0)} {
+                    puts "+ $fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+            foreach id $checkpointIds {
+                if {[string match $pattern $checkpointFullClass($id)]&&([lsearch -exact $currentIds $id]<0)} {
+                    puts "- $checkpointFullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+    }
+
 }
 }
 namespace import stooop::*
@@ -652,7 +723,7 @@ class switched {
 }
 }
 
-if {[catch {package require tkpiechart 5.1}]} {
+if {[catch {package require tkpiechart 5.3}]} {
 set rcsId {$Id: pielabel.tcl,v 1.40 1998/06/07 10:07:30 jfontain Exp $}
 
 class pieLabeler {
@@ -678,7 +749,7 @@ class pieLabeler {
     virtual proc room {this arrayName}
 
 }
-set rcsId {$Id: perilabel.tcl,v 1.45 1998/06/09 21:17:52 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.47 1999/03/24 21:57:40 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -693,7 +764,7 @@ class piePeripheralLabeler {
     }
 
     proc ~piePeripheralLabeler {this} {
-        catch {delete $piePeripheralLabeler::($this,array)}
+        ::delete $piePeripheralLabeler::($this,array)
         $pieLabeler::($this,canvas) delete pieLabeler($this)
     }
 
@@ -789,11 +860,7 @@ class piePeripheralLabeler {
         if {[string length $selected]==0} {
             return $piePeripheralLabeler::($this,selected,$label)
         }
-        if {$selected} {
-            switched::configure $label -borderwidth 2
-        } else {
-            switched::configure $label -borderwidth 1
-        }
+        switched::configure $label -select $selected
         ::set piePeripheralLabeler::($this,selected,$label) $selected
     }
 
@@ -823,7 +890,7 @@ class piePeripheralLabeler {
     }
 
 }
-set rcsId {$Id: boxlabel.tcl,v 1.39 1998/06/07 10:20:43 jfontain Exp $}
+set rcsId {$Id: boxlabel.tcl,v 1.41 1999/03/27 21:41:06 jfontain Exp $}
 
 class pieBoxLabeler {
 
@@ -833,7 +900,7 @@ class pieBoxLabeler {
     }
 
     proc ~pieBoxLabeler {this} {
-        catch {::delete $pieBoxLabeler::($this,array)}
+        ::delete $pieBoxLabeler::($this,array)
     }
 
     proc options {this} {
@@ -878,11 +945,7 @@ class pieBoxLabeler {
         if {[string length $selected]==0} {
             return $pieBoxLabeler::($this,selected,$label)
         }
-        if {$selected} {
-            switched::configure $label -borderwidth 2
-        } else {
-            switched::configure $label -borderwidth 1
-        }
+        switched::configure $label -select $selected
         ::set pieBoxLabeler::($this,selected,$label) $selected
     }
 
@@ -908,7 +971,7 @@ class pieBoxLabeler {
         }
     }
 }
-set rcsId {$Id: canlabel.tcl,v 1.22 1998/06/07 10:08:13 jfontain Exp $}
+set rcsId {$Id: canlabel.tcl,v 1.25 1999/03/30 20:11:01 jfontain Exp $}
 
 class canvasLabel {
 
@@ -916,8 +979,8 @@ class canvasLabel {
         set canvasLabel::($this,canvas) $canvas
         set canvasLabel::($this,origin) [$canvas create image 0 0 -tags canvasLabel($this)]
         set canvasLabel::($this,rectangle) [$canvas create rectangle 0 0 0 0 -tags canvasLabel($this)]
+        set canvasLabel::($this,selectRectangle) [$canvas create rectangle 0 0 0 0 -tags canvasLabel($this)]
         set canvasLabel::($this,text) [$canvas create text 0 0 -tags canvasLabel($this)]
-
         switched::complete $this
     }
 
@@ -931,12 +994,14 @@ class canvasLabel {
             [list -background {} {}]\
             [list -bordercolor black black]\
             [list -borderwidth 1 1]\
-            [list -bulletwidth 20 20]\
+            [list -bulletwidth 10 10]\
             [list -font {Helvetica -12}]\
             [list -foreground black black]\
             [list -justify left left]\
             [list -padding 2 2]\
             [list -scale {1 1} {1 1}]\
+            [list -select 0 0]\
+            [list -selectcolor white white]\
             [list -stipple {} {}]\
             [list -style box box]\
             [list -text {} {}]\
@@ -949,8 +1014,12 @@ class canvasLabel {
     }
     proc set-bordercolor {this value} {
         $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,rectangle) -outline $value
+        if {[string compare $switched::($this,-style) box]==0} {
+            $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,selectRectangle) -outline $value
+        }
     }
     proc set-borderwidth {this value} {
+        $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,selectRectangle) -width $value
         $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,rectangle) -width $value
         update $this
     }
@@ -964,12 +1033,20 @@ class canvasLabel {
         $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,rectangle) -stipple $value
     }
     proc set-style {this value} {
-        if {![regexp {^box|split$} $value]} {
-            error "bad style value \"$value\": must be box or split"
+        switch $value {
+            box {
+                $canvasLabel::($this,canvas) raise $canvasLabel::($this,selectRectangle) $canvasLabel::($this,rectangle)
+            }
+            split {
+                $canvasLabel::($this,canvas) lower $canvasLabel::($this,selectRectangle) $canvasLabel::($this,rectangle)
+            }
+            default {
+                error "bad style value \"$value\": must be box or split"
+            }
         }
         update $this
     }
-    foreach option {-anchor -bulletwidth -padding} {
+    foreach option {-anchor -bulletwidth -padding -select -selectcolor} {
         proc set$option {this value} {update $this}
     }
     foreach option {-font -justify -text -width} {
@@ -982,6 +1059,7 @@ class canvasLabel {
     proc update {this} {
         set canvas $canvasLabel::($this,canvas)
         set rectangle $canvasLabel::($this,rectangle)
+        set selectRectangle $canvasLabel::($this,selectRectangle)
         set text $canvasLabel::($this,text)
 
         foreach {x y} [$canvas coords $canvasLabel::($this,origin)] {}
@@ -989,26 +1067,45 @@ class canvasLabel {
         set border [$canvas itemcget $rectangle -width]
         set textBox [$canvas bbox $text]
         set padding [winfo fpixels $canvas $switched::($this,-padding)]
+        set bulletWidth [winfo fpixels $canvas $switched::($this,-bulletwidth)]
 
-        if {[string compare $switched::($this,-style) split]==0} {
-            set textHeight [expr {[lindex $textBox 3]-[lindex $textBox 1]}]
-            set rectangleWidth [winfo fpixels $canvas $switched::($this,-bulletwidth)]
-            set halfWidth [expr {($rectangleWidth+$padding+([lindex $textBox 2]-[lindex $textBox 0]))/2.0}]
-            set halfHeight [expr {($textHeight/2.0)+$border}]
-            $canvas coords $rectangle\
-                [expr {$x-$halfWidth}] [expr {$y-$halfHeight}] [expr {$x-$halfWidth+$rectangleWidth}] [expr {$y+$halfHeight}]
-            $canvas coords $text [expr {$x+(($rectangleWidth+$padding)/2.0)}] $y
+        $canvas itemconfigure $selectRectangle -fill {} -outline {}
+        set split [expr {[string compare $switched::($this,-style) split]==0}]
+
+        if {$split} {
+            set halfWidth [expr {($bulletWidth+$border+$padding+([lindex $textBox 2]-[lindex $textBox 0]))/2.0}]
+            set halfHeight [expr {(([lindex $textBox 3]-[lindex $textBox 1])/2.0)+$border}]
         } else {
             set width [expr {$switched::($this,-width)==0?[lindex $textBox 2]-[lindex $textBox 0]:$switched::($this,-width)}]
-            set halfWidth [expr {$border+$padding+($width/2.0)}]
-            set halfHeight [expr {$border+$padding+(([lindex $textBox 3]-[lindex $textBox 1])/2.0)}]
-            $canvas coords $rectangle [expr {$x-$halfWidth}] [expr {$y-$halfHeight}] [expr {$x+$halfWidth}] [expr {$y+$halfHeight}]
-            $canvas coords $text $x $y
+            set halfWidth [expr {$bulletWidth+$border+$padding+($width/2.0)}]
+            set halfHeight [expr {(([lindex $textBox 3]-[lindex $textBox 1])/2.0)+$border+$padding}]
         }
+        set left [expr {$x-$halfWidth}]
+        set top [expr {$y-$halfHeight}]
+        set right [expr {$x+$halfWidth}]
+        set bottom [expr {$y+$halfHeight}]
+        $canvas coords $text [expr {$x+(($bulletWidth+$border+$padding)/2.0)}] $y
+        if {$split} {
+            $canvas coords $selectRectangle $left $top $right $bottom
+            $canvas coords $rectangle $left $top [expr {$left+$bulletWidth}] $bottom
+            if {$switched::($this,-select)} {
+                $canvas itemconfigure $selectRectangle\
+                    -fill $switched::($this,-selectcolor) -outline $switched::($this,-selectcolor)
+            }
+        } else {
+            $canvas coords $selectRectangle $left $top [expr {$left+$bulletWidth}] $bottom
+            $canvas coords $rectangle $left $top $right $bottom
+            $canvas itemconfigure $selectRectangle -outline $switched::($this,-bordercolor)
+            if {$switched::($this,-select)} {
+                $canvas itemconfigure $selectRectangle -fill $switched::($this,-selectcolor)
+            }
+        }
+
         set anchor $switched::($this,-anchor)
         set xDelta [expr {([string match *w $anchor]-[string match *e $anchor])*$halfWidth}]
         set yDelta [expr {([string match n* $anchor]-[string match s* $anchor])*$halfHeight}]
         $canvas move $rectangle $xDelta $yDelta
+        $canvas move $selectRectangle $xDelta $yDelta
         $canvas move $text $xDelta $yDelta
         eval $canvas scale canvasLabel($this) $x $y $switched::($this,-scale)
     }
@@ -1343,19 +1440,21 @@ proc slice::data {this arrayName} {
     set data(xCenter) [expr {$x+$data(xRadius)}]
     set data(yCenter) [expr {$y+$data(yRadius)}]
 }
-set rcsId {$Id: selector.tcl,v 1.2 1998/05/24 19:18:21 jfontain Exp $}
+set rcsId {$Id: selector.tcl,v 1.3 1999/01/31 18:47:07 jfontain Exp $}
 
 
 class selector {
 
     proc selector {this args} switched {$args} {
+        ::set selector::($this,order) 0
         switched::complete $this
     }
 
     proc ~selector {this} {
         variable ${this}selected
+        variable ${this}order
 
-        catch {::unset ${this}selected}
+        catch {::unset ${this}selected ${this}order}
     }
 
     proc options {this} {
@@ -1368,6 +1467,7 @@ class selector {
 
     proc set {this indices selected} {
         variable ${this}selected
+        variable ${this}order
 
         ::set select {}
         ::set deselect {}
@@ -1380,6 +1480,8 @@ class selector {
                 lappend deselect $index
                 ::set ${this}selected($index) 0
             }
+            ::set ${this}order($index) $selector::($this,order)
+            incr selector::($this,order)
         }
         update $this $select $deselect
     }
@@ -1396,12 +1498,19 @@ class selector {
 
     proc unset {this indices} {
         variable ${this}selected
+        variable ${this}order
 
         foreach index $indices {
-            ::unset ${this}selected($index)
+            ::unset ${this}selected($index) ${this}order($index)
         }
     }
 
+    proc ordered {this index1 index2} {
+        variable ${this}order
+
+        return [expr {[::set ${this}order($index1)]-[::set ${this}order($index2)]}]
+    }
+
 
     proc add {this indices} {
         set $this $indices 0
@@ -1423,6 +1532,7 @@ class selector {
 
     proc toggle {this indices} {
         variable ${this}selected
+        variable ${this}order
 
         ::set select {}
         ::set deselect {}
@@ -1430,11 +1540,16 @@ class selector {
             if {[::set ${this}selected($index)]} {
                 lappend deselect $index
                 ::set ${this}selected($index) 0
+                if {$index==$selector::($this,lastSelected)} {
+                    ::unset selector::($this,lastSelected)
+                }
             } else {
                 lappend select $index
                 ::set ${this}selected($index) 1
                 ::set selector::($this,lastSelected) $index
             }
+            ::set ${this}order($index) $selector::($this,order)
+            incr selector::($this,order)
         }
         update $this $select $deselect
     }
@@ -1456,17 +1571,18 @@ class selector {
                 lappend list $index
             }
         }
-        return $list
+        return [lsort -command "ordered $this" $list]
     }
 
     virtual proc list {this} {
         variable ${this}selected
 
-        return [array names ${this}selected]
+        return [lsort -command "ordered $this" [array names ${this}selected]]
     }
+
 }
 
-set rcsId {$Id: objselec.tcl,v 1.5 1998/05/24 19:24:43 jfontain Exp $}
+set rcsId {$Id: objselec.tcl,v 1.7 1999/01/31 18:53:01 jfontain Exp $}
 
 
 class objectSelector {
@@ -1478,7 +1594,7 @@ class objectSelector {
 
     proc extend {this id} {
         if {[info exists selector::($this,lastSelected)]} {
-            set list [list $this]
+            set list [lsort -integer [selector::list $this]]
             set last [lsearch -exact $list $selector::($this,lastSelected)]
             set index [lsearch -exact $list $id]
             selector::clear $this
@@ -1492,17 +1608,10 @@ class objectSelector {
         }
     }
 
-    proc selected {this} {
-        return [lsort -integer [selector::_selected $this]]
-    }
-
-    proc list {this} {
-        return [lsort -integer [selector::_list $this]]
-    }
 }
-set rcsId {$Id: pie.tcl,v 1.80 1998/06/09 21:20:45 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.85 1999/03/27 21:41:51 jfontain Exp $}
 
-package provide tkpiechart 5.2
+package provide tkpiechart 5.3
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
@@ -1518,10 +1627,14 @@ proc pie::pie {this canvas x y args} switched {$args} {
 }
 
 proc pie::~pie {this} {
-    catch {$pie::($this,canvas) delete $pie::($this,title)}
+    if {[info exists pie::($this,title)]} {
+        $pie::($this,canvas) delete $pie::($this,title)
+    }
     delete $pie::($this,labeler)
     eval delete $pie::($this,slices) $pie::($this,backgroundSlice)
-    catch {delete $pie::($this,selector)}
+    if {[info exists pie::($this,selector)]} {
+        delete $pie::($this,selector)
+    }
     $pie::($this,canvas) delete $pie::($this,origin)
 }
 
@@ -1712,7 +1825,7 @@ proc pie::setLabelsState {this labels selected} {
 
 proc pie::currentSlice {this} {
     set tags [$pie::($this,canvas) gettags current]
-    if {[scan $tags slice(%u) slice]>0} {
+    if {([scan $tags slice(%u) slice]>0)&&($slice!=$pie::($this,backgroundSlice))} {
         return $slice
     }
     if {[scan $tags canvasLabel(%u) label]>0} {
