@@ -1,7 +1,7 @@
-# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
+# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: objselec.tcl,v 1.5 1998/05/24 19:24:43 jfontain Exp $}
+set rcsId {$Id: objselec.tcl,v 1.7 1999/01/31 18:53:01 jfontain Exp $}
 
 # implements selection on a list of object identifiers (sortable list of integer), for a listbox implementation, for example
 
@@ -15,7 +15,7 @@ class objectSelector {
 
     proc extend {this id} {
         if {[info exists selector::($this,lastSelected)]} {
-            set list [list $this]
+            set list [lsort -integer [selector::list $this]]
             set last [lsearch -exact $list $selector::($this,lastSelected)]
             set index [lsearch -exact $list $id]
             selector::clear $this
@@ -29,11 +29,4 @@ class objectSelector {
         }
     }
 
-    proc selected {this} {
-        return [lsort -integer [selector::_selected $this]]
-    }
-
-    proc list {this} {
-        return [lsort -integer [selector::_list $this]]
-    }
 }
