@@ -1,4 +1,4 @@
-set rcsId {$Id: perilabel.tcl,v 1.45 1998/06/09 21:17:52 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.47 1999/03/24 21:57:40 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -13,8 +13,8 @@ class piePeripheralLabeler {
     }
 
     proc ~piePeripheralLabeler {this} {
-        catch {delete $piePeripheralLabeler::($this,array)}                                   ;# array may not have been created yet
-        $pieLabeler::($this,canvas) delete pieLabeler($this)                                               ;# delete remaining items
+        ::delete $piePeripheralLabeler::($this,array)
+        $pieLabeler::($this,canvas) delete pieLabeler($this)          ;# delete remaining items (should be in pieLabeler destructor)
     }
 
     proc options {this} {                        ;# bullet width, font and justify options are used when creating a new canvas label
@@ -113,11 +113,7 @@ class piePeripheralLabeler {
         if {[string length $selected]==0} {                                                   ;# return current state if no argument
             return $piePeripheralLabeler::($this,selected,$label)
         }
-        if {$selected} {
-            switched::configure $label -borderwidth 2
-        } else {
-            switched::configure $label -borderwidth 1
-        }
+        switched::configure $label -select $selected
         ::set piePeripheralLabeler::($this,selected,$label) $selected
     }
 
